/*
 * augment.c,v 1.1.1.1 1996/03/03 17:39:50 arsharma Exp
 * Purpose : To write out augmentation into the output file
 */

#include <stdio.h>
#include <stddef.h>
#include <assert.h>

#include "doctor_t.h"
#include "lex.h"
#include "memref.h"
#include "instr.h"
#include "mnemdef.h"
#include "event.h"
#include "internal.h"
#include "augment.h"
#include "register.h"

extern bool_t default_data_movement, count_references;
extern char *aug_buffer;
extern int aug_end;

/* This becomes necessary in the stack_offset cases */
static char operand[MAX_OP_SIZE];

/* called only if instr has a memory reference */
void
augment_instruction (instr_t *instr)
{
    unsigned mnemonic_code;

    mnemonic_code = instr->mnemonic->representation;

    /* figure out if this is a floating point instruction */
    instr->floating_point = (mnemonic_code >= F2XM1)
	&& (mnemonic_code <= FYL2XP1);

    check_references (instr);
}

/*
 * Writes out the augmentation that is appropriate for the
 * instruction.
 * Please refer to Alog.data_movement for a detailed description of
 * the algorithm used for the data movement
 */
void
check_references (instr_t *instr)
{
    ref_t *ref1, *ref2;
    bool_t data_movement;

    ref1 = instr->mem_ref[0];
    ref2 = instr->mem_ref[1];

    data_movement = data_movement_safe (instr);

    /* case 1 there's only one reference */
    if ((ref1) && not (ref2)) {
	if (ref1->read) {

	    augment ("; A read\n");

	    if (count_references) {
		augment ("\tinc read_ref\n");
		augment ("\tinc count_ref\n");
	    }
	    else if (data_movement) {
		augment_push_regs ();
		augment_put_stuff (instr, ref1);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
		augment_push_regs ();
		augment_move_data (instr, ref1);
		augment_pop_regs ();
		augment_done (instr);
		/* The original instruction goes here */
		augment_push_regs ();
		augment_restore_memory (instr);
		augment_pop_regs ();
	    }
	    else {
		/* normal case */
		augment_push_regs ();
		augment_put_stuff (instr, ref1);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
	    }
	}
	else {
	    /* if ref1->write */
	    augment ("; A write\n");

	    if (count_references) {
		augment ("\tinc write_ref\n");
		augment ("\tinc count_ref\n");
	    }
	    else if (data_movement) {
		augment_push_regs ();
		/* save the memory location */
		augment_move_data (instr, ref1);
		augment_pop_regs ();
		augment_done (instr);
		/* The original instruction goes here */
		augment_push_regs ();
		augment_put_stuff (instr, ref1);
		augment_put_more_stuff (instr, ref1);
		augment_restore_memory (instr);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
	    }
	    else {
		/* normal case */
		augment_push_regs ();
		augment_put_stuff (instr, ref1);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
	    }
	}
    }
    else if (ref1 && ref2) {
	/* case 2 there are two references, both reads */
	if ((ref1->read) && (ref2->read)) {

	    augment ("; two reads\n");

	    if (count_references) {
		augment ("\tinc read_ref\n");
		augment ("\tinc read_Ref\n");
		augment ("\tinc count_ref\n");
		augment ("\tinc count_ref\n");
	    }
	    else if (data_movement) {
		augment_push_regs ();
		augment_put_stuff (instr, ref1);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
		augment_push_regs ();
		augment_move_data (instr, ref1);
		/* no popping here */
		augment_put_stuff (instr, ref2);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
		augment_push_regs ();
		augment_move_data (instr, ref2);
		augment_pop_regs ();
		augment_done (instr);
		/* The original instruction goes here */
		augment_push_regs ();
		augment_restore_memory (instr);
		augment_pop_regs ();
	    }
	    else {
		/* normal case */
		augment_push_regs ();
		augment_put_stuff (instr, ref1);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
		augment_push_regs ();
		augment_put_stuff (instr, ref2);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
	    }
	}
	else if ((ref1->read) && (ref2->write)) {
	    /* case 3 there are two references, one read and one write */
	    augment ("; A read and a write\n");

	    if (count_references) {
		augment ("\tinc read_ref\n");
		augment ("\tinc write_ref\n");
		augment ("\tinc count_ref\n");
		augment ("\tinc count_ref\n");
	    }
	    else if (data_movement) {
		augment_push_regs ();
		augment_put_stuff (instr, ref1);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
		augment_push_regs ();
		augment_move_data (instr, ref1);
		augment_pop_regs ();
		augment_push_regs ();
		augment_move_data (instr, ref2);
		augment_pop_regs ();
		augment_done (instr);
		/* The original instruction goes here */
		augment_push_regs ();
		augment_put_stuff (instr, ref2);
		augment_put_more_stuff (instr, ref2);
		/* restore everything - we need to recompute the vaddr */
		augment_pop_regs ();
		augment_push_regs ();
		augment_restore_memory (instr);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
	    }
	    else {
		/* normal case */
		augment_push_regs ();
		augment_put_stuff (instr, ref1);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
		augment_push_regs ();
		augment_put_stuff (instr, ref2);
		augment_pop_regs ();
		augment_ctxsw_intel (instr);
	    }
	}
    }
    else
	/* we have a new case here */
	assert (0);
}

void
augment_push_regs ()
{
    /* let's find an efficient way later */
    augment ("\tpushf\n");
    augment ("\tpush eax\n");
    augment ("\tpush ebx\n");
}

char *unused_reg;

/*
 * puts	      data into pcur_thread->pevent structure
 * Specifically, updates the type,size,vaddr fields
 */
void
augment_put_stuff (instr_t *instr, ref_t *ref)
{
    /*
     * move the	      virtual address into %eax. This is done without
     * disturbing any of the registers, to ensure that the right virtual
     * address is loaded into %eax
     */

    /* get the correct stack address */
    if (ref->stack) {
	if (ref->read) {
	    sprintf (operand, ref->format_string, ref->stack_offset);
	    ref->source = operand;
	}
	else if (ref->write) {
	    /* It's a write */
	    sprintf (operand, ref->format_string, ref->stack_offset);
	    ref->destination = operand;
	}
    }

    augment ("\tlea eax,");
    if (ref->read)
	augment ("%s\n", ref->source);
    else
	augment ("%s\n", ref->destination);

    /* get the offset of the `pevent' field into %ebx */
    GET_PEVENT (ebx);

    /* set the event type (r/w) */
    if (ref->read)
	augment ("\tmov	 DWORD PTR [ebx+%d],%d\n",
			 EVENT_TYPE_OFFSET,
			 instr->lock ? E_LOCK_READ : E_READ | E_READ_OFFSET);
    else if (ref->write)
	augment ("\tmov DWORD PTR[ebx+%d],%d\n",
			 EVENT_TYPE_OFFSET,
			 instr->lock ? E_LOCK_WRITE : E_WRITE | E_WRITE_OFFSET);

    /* move the operand size into the `size' field */
    augment ("\tmov DWORD PTR[ebx+%d],%d\n",
	     EVENT_SIZE_OFFSET,instr->mnemonic->operand_size);

    /* move virtual address into the `vaddr' field */
    augment ("\tmov DWORD PTR[ebx+%d],eax\n", EVENT_VADDR_OFFSET);

    /* Update the thread clock */
    augment_safe_update_clock ();
}

void
augment_pop_regs ()
{
    augment ("\tpop ebx\n");
    augment ("\tpop eax\n");
    augment ("\tpopf \n");
}

/* writes out code to do the context switch */
void
augment_ctxsw_intel (instr_t *instr)
{
    augment ("\tpush _pcur_thread\n");
    augment ("\tpush _psim_thread\n");
    augment ("\tpush _pcur_thread\n");
    augment ("\tcall _ctxsw_intel\n");
    augment ("\tadd  esp,12\n");
}

/* For writes, we pass the `value' too to the backend */
void
augment_put_more_stuff (instr_t *instr, ref_t *ref)
{
    if (ref->write) {
	/* move the value into pevent.value */
	augment ("\tpush DWORD PTR [eax]\n");
	augment ("\tpop DWORD PTR [ebx+%d]\n", EVENT_VALUE_OFFSET);
	if (instr->double_data) {
	    augment ("\tpush DWORD PTR [eax+4]\n");
	    augment ("\tpop DWORD PTR [ebx+%d]\n", EVENT_VALUE_OFFSET + 4);
	}
    }
}

/*
 * Saves the memory locations involved and moves the data to/from the
 * backend
 */
void
augment_move_data (instr_t *instr, ref_t *ref)
{
    if (ref->write) {

	/* get the stack virtual address correctly */
	if (ref->stack) {
	    sprintf (operand, ref->format_string, ref->stack_offset);
	    ref->destination = operand;
	}

	/* get the vaddr in %eax */
	augment ("\tlea eax,%s\n", ref->destination);

	/* if ref is the first reference */
	if (instr->mem_ref[0] == ref) {
	    /* save the memory location in tmp1 */
	    augment ("\tpush ebx\n");
	    augment ("\tpush DWORD PTR [eax]\n");
	    GET_PTHREAD (ebx);
	    augment ("\tpop DWORD PTR [ebx+%d]\n", THREAD_TMP1_OFFSET);
	    if (instr->double_data) {
		augment ("\tpush DWORD PTR [eax+4]\n");
		augment ("\tpop DWORD PTR [ebx+%d]\n", THREAD_TMP1_OFFSET + 4);
	    }
	    augment ("\tpop ebx\n");
	}
	else {
	    /* if it is the second, use tmp2 */
	    augment ("\tpush ebx\n");
	    augment ("\tpush DWORD PTR [eax]\n");
	    GET_PTHREAD (ebx);
	    augment ("\tpop DWORD PTR [ebx+%d]\n", THREAD_TMP2_OFFSET);
	    if (instr->double_data) {
		augment ("\tpush DWORD PTR [eax+4]\n");
		augment ("\tpop DWORD PTR [ebx+%d]\n", THREAD_TMP2_OFFSET + 4);
	    }
	    augment ("\tpop ebx\n");
	}
    }
    else if (ref->read) {

	GET_PEVENT (ebx);

	/* move the vaddr into eax */
	augment ("\tmov DWORD PTR [ebx+%d],eax\n", EVENT_VADDR_OFFSET);

	if (instr->mem_ref[0] == ref) {
	    /* save the memory location in tmp1 */
	    augment ("\tpush ebx\n");
	    augment ("\tpush DWORD PTR [eax]\n");
	    GET_PTHREAD (ebx);
	    augment ("\tpop DWORD PTR [ebx+%d]\n", THREAD_TMP1_OFFSET);
	    if (instr->double_data) {
		augment ("\tpush DWORD PTR [eax+4]\n");
		augment ("\tpop DWORD PTR [ebx+%d]\n", THREAD_TMP1_OFFSET + 4);
	    }
	    augment ("\tpop ebx\n");
	}
	else {
	    /* use tmp2 */
	    augment ("\tpush ebx\n");
	    augment ("\tpush DWORD PTR [eax]\n");
	    GET_PTHREAD (ebx);
	    augment ("\tpop DWORD PTR [ebx+%d]\n", THREAD_TMP2_OFFSET);
	    if (instr->double_data) {
		augment ("\tpush DWORD PTR [eax+4]\n");
		augment ("\tpop DWORD PTR [ebx+%d]\n", THREAD_TMP2_OFFSET + 4);
	    }
	    augment ("\tpop ebx\n");
	}

	/* move pevent.value into ebx and then into (mem) */
	switch (instr->mnemonic->operand_size) {
	    case 1:
		augment ("\tmov bl,BYTE PTR [ebx+%d]\n", EVENT_VALUE_OFFSET);
		augment ("\tmov BYTE PTR [eax],bl\n");
		break;
	    case 2:
		augment ("\tmov bx,WORD PTR [ebx+%d]\n", EVENT_VALUE_OFFSET);
		augment ("\tmov WORD PTR [eax],bx\n");
		break;
	    case 4:
		augment ("\tmov ebx,DWORD PTR [ebx+%d]\n", EVENT_VALUE_OFFSET);
		augment ("\tmov DWORD PTR [eax],ebx\n");
		break;
	    case 8:
		augment ("\tpush ebx\n");
		augment ("\tmov ebx,DWORD PTR [ebx+%d]\n", EVENT_VALUE_OFFSET);
		augment ("\tmov DWORD PTR [eax],ebx\n");
		augment ("\tpop ebx\n");
		augment ("\tmov ebx,DWORD PTR [ebx+%d]\n", EVENT_VALUE_OFFSET + 4);
		augment ("\tmov DWORD PTR [eax+4],ebx\n");
		break;
	    default:
		assert (0);
		break;
	}
    }
}

/* restores the saved memory locations, after the data movement */
void
augment_restore_memory (instr_t *instr)
{
    ref_t *ref1, *ref2;

    ref1 = instr->mem_ref[0];
    ref2 = instr->mem_ref[1];

    if ((ref1->read) || (ref1->write)) {

	if (ref2) {
	    /* get the correct stack offset */
	    if (ref1->stack) {
		if (ref1->read) {
		    sprintf (operand, ref1->format_string,
			     ref1->stack_offset);
		    ref1->source = operand;
		}
		else if (ref1->write) {
		    /* It's a write */
		    sprintf (operand, ref1->format_string,
			     ref1->stack_offset);
		    ref1->destination = operand;
		}
	    }

	    /* get the	`vaddr' */
	    augment ("\tlea eax,");
	    if (ref1->read)
		augment ("%s\n", ref1->source);
	    else
		augment ("%s\n", ref1->destination);
	}
	else if (ref1->read) {
	    /*
	     * In the case of a	      write, we already have ebx and eax setup
	     * properly, so this part is unnecessary
	     */

	    /* get pevent in ebx */
	    GET_PEVENT (ebx);

	    /* move the vaddr into eax */
	    augment ("\tmov eax,DWORD PTR [ebx+%d]\n", EVENT_VADDR_OFFSET);
	}
	/* restore the memory location from tmp1 */
	augment ("\tpush ebx\n");
	GET_PTHREAD (ebx);
	augment ("\tpush DWORD PTR [ebx+%d]\n", THREAD_TMP1_OFFSET);
	augment ("\tpop DWORD PTR [eax]\n");
	if (instr->double_data) {
	    augment ("\tpush DWORD PTR [ebx+%d]\n", THREAD_TMP1_OFFSET + 4);
	    augment ("\tpop DWORD PTR [eax+4]\n");
	}
	augment ("\tpop ebx\n");
    }

    if (ref2 && ((ref2->read) || (ref2->write))) {

	/* get pevent in ebx */
	GET_PEVENT (ebx);

	/* move the vaddr into eax */
	augment ("\tmov eax,DWORD PTR [ebx+%d]\n", EVENT_VADDR_OFFSET);

	/* restore the memory location from tmp2 */
	augment ("\tpush ebx\n");
	GET_PTHREAD (ebx);
	augment ("\tpush DWORD PTR [ebx+%d]\n", THREAD_TMP2_OFFSET);
	augment ("\tpop DWORD PTR [eax]\n");
	if (instr->double_data) {
	    augment ("\tpush DWORD PTR [ebx+%d]\n", THREAD_TMP2_OFFSET + 4);
	    augment ("\tpop DWORD PTR [eax+4]\n");
	}
	augment ("\tpop ebx\n");
    }

}

/*
 * Inserts augmentation	      for updating the thread time. The counting is
 * done according to the `base' time specified in the pentium manual.
 *
 * Warning: Assumes the ebx has pcur_thread, when it is called.
 */

void
augment_update_clock ()
{
    if (static_execution_time) {

		augment ("\tmov _static_execution_time,%d\n", static_execution_time);
		augment ("\tfld QWORD PTR [ebx+%d]\n", THREAD_TIME_OFFSET);
		augment ("\tfiadd _static_execution_time\n");
		augment ("\tfstp QWORD PTR [ebx+%d]\n", THREAD_TIME_OFFSET);

		/* reset the time to zero */
		static_execution_time = 0;
    }
}

/*
 * This	      one is the same as augment_update_clock, but doesn't assume
 * anything about ebx.
 */

void
augment_safe_update_clock ()
{
    if (static_execution_time) {
	augment_push_regs ();
	GET_PTHREAD (ebx);
	augment ("\tmov _static_execution_time,%d\n", static_execution_time);
	augment ("\tfld QWORD PTR [ebx+%d]\n", THREAD_TIME_OFFSET);
	augment ("\tfiadd _static_execution_time\n");
	augment ("\tfstp QWORD PTR [ebx+%d]\n", THREAD_TIME_OFFSET);
	augment_pop_regs ();

	/* reset the time to zero */
	static_execution_time = 0;
    }
}

/* checks if it is safe to do the data movement */
bool_t
data_movement_safe (instr_t *instr)
{
    /* indirect jumps are not handled */
    if ((instr->mnemonic->representation == JMP)
	&& (instr->modes[0] == INDIRECT))
	return FALSE;

    /* sizes greater than 4 bytes are not handled */
    if ((instr->mnemonic->operand_size > 8))
	return FALSE;

    return default_data_movement;
}
