/*
 * doctor.l,v 1.1.1.1 1996/03/03 17:39:50 arsharma Exp
 *
 * This is the input to the lexical analyzer
 * While the characters are being read, they are also stuffed into the
 * 'output buffer' and flushed at the end of basic blocks
 */
%{

#include "doctor_t.h"
#include "keyword_.h"
#include  "mnemdef.h"
#include "lex.h"
#include "symtab.h"
#include "memref.h"
#include "instr.h"
#include "write.h"

#include <stdio.h>
#include <ctype.h>
#include <varargs.h>

extern unsigned start_first, end_first, start_second, end_second;

int seen_mnemonic = FALSE;

%}

White_space     [ \t]
Hex_digit       [0-9a-fA-F]
Digit           [0-9]
Letter          [a-zA-Z_?@$]
Letter_Or_Digit [a-zA-Z_?@$0-9]
Label_letter    [a-zA-Z_0-9.]
Label_quirks    [^\n,]
Asterisk        [*]
Slash           [/]
Other           .
Non_colon       [^:\n]
String          "[^\n"]*"

/* Just to fool the syntax highlighter of emacs " */

%%

[ \t]                   {
	/* Do nothing white space */
	token_store();
}

[\n]                    {
    token_store();
    seen_mnemonic = FALSE;
    end_second = 0;
    return NEWLINE;
}

";"{Other}*                    {
    token_store();
    return COMMENT;
}

","                            {
    end_first = save_pos();
	token_store();
    start_second = save_pos();
	return COMMA;
}

{Digit}{Digit}*    {
	token_store();
	return CONSTANT;
}

^{White_space}*"."{Other}* {
	/*
	 * every line, that     begins with a
	 * DOT and anything     other than a
	 * `L' is a     directive
	 */
	token_store();
	return DIRECTIVE;
}

^{Other}*"_aug_on_"{Other}* {
	return AUG_ON;
}

^{Other}*"_aug_off_"{Other}* {
	return AUG_OFF;
}

"."                            {
	token_store();
	return DOT;
}

{Asterisk}                     {
	token_store();
	return ASTERISK;
}

^{Letter}({Letter}|{Digit})*":"    {
	token_store();
	return LABEL;
}

":"                            {
	token_store();
	return COLON;
}

"("                            {
	token_store();
	return OB;
}

")"                            {
	token_store();
	return CB;
}

"["                            {
	token_store();
	return SQOB;
}

"]"                            {
	token_store();
	return SQCB;
}

"+"                            {
	token_store();
	return PLUS;
}

"-"                            {
	token_store();
	return MINUS;
}

.                       {
    /* ' */
    token_store();
    return JUNK;
}

{Letter}({Letter}|{Digit})*    {
	token_store();
	return screen();
}

%%


/* look  in lex.h for an explanation of the various fields */
static mnemonic_table_t mnemonic_table[] =
{
#include "mnemtab.h"
};

char last_id[MAX_ID_LENGTH];
extern unsigned start_pos, start_instr;
mnemonic_table_t *last_mnem;
/*
 * This       routine checks if the yytext is a keyword. If so, it returns
 * the enum const corresponding to the keyword. Otherwise it checks if
 * it is a mnemonic, if it is,  it returns MNEMONIC. Otherwise it
 * returns ID
 */

static int
screen ()
{
    struct keyword_table
     *low = keyword_table, *high = LAST (keyword_table), *mid;

    struct mnemonic_table
     *low_mnem = mnemonic_table, *high_mnem = LAST (mnemonic_table), *mid_mnem;

    int c = 0;
    char lower_case[MAX_ID_LENGTH];

    while (yytext[c])
	lower_case[c++] = tolower (yytext[c]);

    lower_case[c] = '\0';

    while (low <= high) {
	mid = low + (high - low) / 2;

	if (!(c = strcmp (mid->keyword_name, (char *) lower_case)))
	    return mid->keyword_representation;
	else if (c < 0)
	    low = mid + 1;
	else
	    high = mid - 1;
    }

    /*
     * check if       it is a mnemonic if we haven't seen one already on the
     * current line
     */
    if (!seen_mnemonic)
		while (low_mnem <= high_mnem) {
	    mid_mnem = low_mnem + (high_mnem - low_mnem) / 2;

	    if (!(c = strcmp (mid_mnem->mnemonic_name, lower_case))) {
		last_mnem = mid_mnem;
		seen_mnemonic = TRUE;

		/*
		 * The beginning of this line should be       the beginning of the
		 * instruction
		 */
		start_instr = start_line ();

		/*
		 * The end of the mnemonic is the start       of the first
		 * operand
		 */
		start_first = save_pos ();

		return MNEMONIC;
	    }
	    else if (c < 0)
		low_mnem = mid_mnem + 1;
	    else
		high_mnem = mid_mnem - 1;
	}

    return ID;
}

yywrap ()
{
    /* donno what this does */
    return 1;
}

/* all errors lead to this function */
yyerror (va_alist)
     va_dcl
{
    va_list argp;
    char *fmt;
    char message[MAX_ID_LENGTH * 2];

    va_start (argp);
    fmt = va_arg (argp, char *);
    vsprintf (message, fmt, argp);
    yywhere ();
    fprintf (stderr, "%s \n", message);

    exit (1);
}

/* where are we ? */
yywhere ()
{
    extern char input_filename[MAX_ID_LENGTH];

    if (strlen (input_filename))
	fprintf (stderr, "%s:%d: ", input_filename, yylineno);
}

/* store the current token in the buffer - char by char */
token_store ()
{
    char c;
    int i = 0;

    while ((c = yytext[i++]) != 0)
	put_buf (c);
}
