%{

    /*
     * doctor.y,v 1.1.1.1 1996/03/03 17:39:50 arsharma Exp
     * This is the input to bison and has the parser
     * The grammar has been taken from the microsoft assembler manual
     */

#include "doctor_t.h"
#include "mnemdef.h"
#include "lex.h"
#include "symtab.h"
#include "memref.h"
#include "instr.h"
#include "write.h"
#include "register.h"

#include <stdio.h>
extern char yytext[];
enum addr_mode;

/* This section has all the global variables used by doctor */

/*
 * Index into the output buffer, pointing to the end of       the previous
 * instruction
 */
unsigned start_pos;

/* See the documentation for instr_t->start_instr in instr.h */
unsigned start_instr;

/* indicates the start and end of the first operand  */
unsigned start_first, end_first;

/*
 * indicates the start and end of the second operand in       the output
 * buffer
 */
unsigned start_second, end_second;

/*
 * Indicates the number       of the current instruction,
 * within the basic block
 */
unsigned instr_num = 0;

/* used for register usage analysis */
unsigned reg_mask = 0;

/*
 * used       to count the static clock cycles since the last update
 * Reset when the thread clock is updated
 */
unsigned static_execution_time = 0;

/* number of operands in an instruction */
int num_ops;

/*
 * this       is an array of pointers, each one pointing to a record,
 * corresponding to an asm instruction
 */
instr_t **basic_block;

instr_t *current;               /* the current instruction */

/* True if we're at the end of a basic block */
bool_t end_block;

/* True if we're doing verification */
bool_t default_data_movement = FALSE;

/* True if we're just counting the memory references i.e. -c flag */
bool_t count_references = FALSE;

/* Toggled by aug_on and aug_off */
bool_t augment_on = TRUE;

/* Augment even stack references */
bool_t stack_ref_augment = TRUE;

/* Instructions normally don't have locks */
bool_t lock = FALSE;

/* no repeat prefix */
bool_t repeat_prefix = FALSE;

/* For NT only */

/* size of the regs used */
int     reg_size;

/* word ptr, byte ptr etc */
int     pointer_size;

%}

%token ABS
%token ADDR
%token AH
%token AL
%token ALIGN
%token ASSUME
%token ASTERISK
%token AT
%token AUG_ON
%token AUG_OFF
%token AX
%token BASIC
%token BCDCONST
%token BH
%token BL
%token BOGUS
%token BP
%token BX
%token BYTE
%token C
%token CB
%token CH
%token CL
%token COLON
%token COMM
%token COMMA
%token COMMENT
%token COMMENTDIR
%token COMMON
%token CONSTANT
%token CR0
%token CR2
%token CR3
%token CS
%token CX
%token DB
%token DD
%token DF
%token DGROUP
%token DH
%token DI
%token DIRECTIVE
%token DL
%token DO
%token DOLLAR
%token DOSSEG
%token DOT
%token DOT_TYPE
%token DOUBLE_AT
%token DOUBLE_COLON
%token DQ
%token DR0
%token DR1
%token DR2
%token DR3
%token DR6
%token DR7
%token DS
%token DT
%token DUP
%token DW
%token DWORD
%token DX
%token EAX
%token EBP
%token EBX
%token ECHODIR
%token ECX
%token EDI
%token EDX
%token ELSE
%token ELSEIF
%token ELSEIF1
%token ELSEIF2
%token ELSEIFB
%token ELSEIFDEF
%token ELSEIFDIF
%token ELSEIFDIFI
%token ELSEIFE
%token ELSEIFIDNI
%token ELSEIFNB
%token ELSEIFNDEF
%token END
%token ENDIF
%token ENDP
%token ENDS
%token EQ
%token EQU
%token EQUALS
%token ERROR
%token ES
%token ESI
%token ESP
%token EVEN
%token EXPORT
%token EXTERN
%token EXTERNDEF
%token EXTRN
%token FAR
%token FAR16
%token FAR32
%token FLAT
%token FLOATNUMBER
%token FORTRAN
%token FS
%token FWORD
%token GE
%token GROUP
%token GS
%token GT
%token HIGH
%token HIGHWORD
%token ID
%token IF
%token IF1
%token IF2
%token IFB
%token IFDEF
%token IFDIF
%token IFDIFI
%token IFE
%token IFIDNI
%token IFNB
%token IFNDEF
%token INCLUDE
%token INCLUDELIB
%token INVOKE
%token JUNK
%token LABEL
%token LE
%token LENGTH
%token LENGTHOF
%token LOCAL
%token LOCK
%token LOW
%token LOWWORD
%token LROFFSET
%token LT
%token MASK
%token MEMORY
%token MINUS
%token MNEMONIC
%token MOD
%token NAME
%token NE
%token NEAR
%token NEAR16
%token NEAR32
%token NEWLINE
%token NOTHING
%token NUMBER
%token OB
%token OFFSET
%token OPATTR
%token OPTION
%token ORG
%token PAGE
%token PARA
%token PASCAL
%token PERCENT
%token PLUS
%token PRIVATE
%token PROC
%token PROTO
%token PTR
%token PUBLIC
%token PURGE
%token QUESTION_MARK
%token QWORD
%token READONLY
%token REAL10
%token REAL4
%token REAL8
%token REP
%token REPE
%token REPNE
%token REPNZ
%token REPZ
%token SBYTE
%token SDWORD
%token SEG
%token SEGMENT
%token SHORT
%token SI
%token SIZE
%token SIZEOF
%token SLASH
%token SP
%token SQCB
%token SQOB
%token SS
%token ST
%token STACK
%token STDCALL
%token STRING
%token SWORD
%token SYSCALL
%token TBYTE
%token TEXTLITERAL
%token THIS
%token TITLEDIR
%token TR3
%token TR4
%token TR5
%token TR6
%token TR7
%token TYPE
%token USE16
%token USE32
%token USES
%token VARARG
%token WIDTH
%token WORD

%union {
    enum addr_modes y_mod; /* addressing mode */
    enum addr_modes *y_mods;
    int  y_int;
}

%type <y_mod>  e01 e02 e03 expr  disp address
%type <y_mods> asmInstruction

%%


program         : /* nothing */
		|  program line

line            : directive tail
		| labeldef
		  {
		      end_block = TRUE;
		      if (current) {
			  current->end_block = TRUE;
			  flush_basic_block();
		      }
		  }
		| tail
		| instruction tail
		  {
			pointer_size = 0;
			reg_size = 0;
		  }
		| masm_junk tail
		| AUG_ON
		  {
		      augment_on = TRUE;
		  }
		| AUG_OFF
		  {
		      augment_on = FALSE;
		  }
		;

non_instruction : ABS
		| ADDR
		| AH
		| AL
		| ALIGN
		| ASSUME
		| ASTERISK
		| AT
		| AX
		| BASIC
		| BCDCONST
		| BH
		| BL
		| BOGUS
		| BP
		| BX
		| BYTE
		| C
		| CB
		| CH
		| CL
		| COLON
		| COMM
		| COMMA
		| COMMENT
		| COMMENTDIR
		| COMMON
		| CONSTANT
		| CR0
		| CR2
		| CR3
		| CS
		| CX
		| DB
		| DD
		| DF
		| DGROUP
		| DH
		| DI
		| DIRECTIVE
		| DL
		| DO
		| DOLLAR
		| DOSSEG
		| DOT
		| DOT_TYPE
		| DOUBLE_AT
		| DOUBLE_COLON
		| DQ
		| DR0
		| DR1
		| DR2
		| DR3
		| DR6
		| DR7
		| DS
		| DT
		| DUP
		| DW
		| DWORD
		| DX
		| EAX
		| EBP
		| EBX
		| ECHODIR
		| ECX
		| EDI
		| EDX
		| ELSE
		| ELSEIF
		| ELSEIF1
		| ELSEIF2
		| ELSEIFB
		| ELSEIFDEF
		| ELSEIFDIF
		| ELSEIFDIFI
		| ELSEIFE
		| ELSEIFIDNI
		| ELSEIFNB
		| ELSEIFNDEF
		| END
		| ENDIF
		| ENDP
		| ENDS
		| EQ
		| EQU
		| EQUALS
		| ERROR
		| ES
		| ESI
		| ESP
		| EVEN
		| EXPORT
		| EXTERN
		| EXTERNDEF
		| EXTRN
		| FAR
		| FAR16
		| FAR32
		| FLAT
		| FLOATNUMBER
		| FORTRAN
		| FS
		| FWORD
		| GE
		| GROUP
		| GS
		| GT
		| HIGH
		| HIGHWORD
		| ID
		| IF
		| IF1
		| IF2
		| IFB
		| IFDEF
		| IFDIF
		| IFDIFI
		| IFE
		| IFIDNI
		| IFNB
		| IFNDEF
		| INCLUDE
		| INCLUDELIB
		| INVOKE
		| JUNK
		| LE
		| LENGTH
		| LENGTHOF
		| LOCAL
		| LOCK
		| LOW
		| LOWWORD
		| LROFFSET
		| LT
		| MASK
		| MEMORY
		| MINUS
		| MOD
		| NAME
		| NE
		| NEAR
		| NEAR16
		| NEAR32
		| NOTHING
		| NUMBER
		| OB
		| OFFSET
		| OPATTR
		| OPTION
		| ORG
		| PAGE
		| PARA
		| PASCAL
		| PERCENT
		| PLUS
		| PRIVATE
		| PROC
		| PROTO
		| PTR
		| PUBLIC
		| PURGE
		| QUESTION_MARK
		| QWORD
		| READONLY
		| REAL10
		| REAL4
		| REAL8
		| REP
		| REPE
		| REPNE
		| REPNZ
		| REPZ
		| SBYTE
		| SDWORD
		| SEG
		| SEGMENT
		| SHORT
		| SI
		| SIZE
		| SIZEOF
		| SLASH
		| SP
		| SQCB
		| SQOB
		| SS
		| ST
		| STACK
		| STDCALL
		| STRING
		| SWORD
		| SYSCALL
		| TBYTE
		| TEXTLITERAL
		| THIS
		| TITLEDIR
		| TR3
		| TR4
		| TR5
		| TR6
		| TR7
		| TYPE
		| USE16
		| USE32
		| USES
		| VARARG
		| WIDTH
		| WORD
		;

anything        : all_tokens
		| anything all_tokens
		| /* null */
		;

masm_junk       : non_instruction anything
		;

all_tokens      : non_instruction
		| MNEMONIC
		;

labeldef        : LABEL
		;

tail            : opt_comment NEWLINE;

instruction     : opt_instrPrefix asmInstruction
		  {
		    extern int yylineno;
		    extern mnemonic_table_t *last_mnem;

		    /* allocate memory for the current instr */
		    current = new_instruction();

		    /* put info into it */
		    current->source_line = yylineno;
		    current->modes = $2;
		    current->start_text = start_pos;
		    current->end_text = save_pos();
		    current->start_instr = start_instr;
		    current->start_first = start_first;
		    current->end_first = end_first;
		    current->start_second = start_second;
		    current->end_second = end_second;
		    current->mnemonic = last_mnem;
		    current->reg_mask = reg_mask;
		    current->num_ops = num_ops;
		    current->augment = augment_on;
		    current->lock = lock;
		    current->rep = repeat_prefix;
		    /*
		     * this flag tells if the next instruction is a
		     * new basic block
		     */
		    end_block =
		      isjumper(current->mnemonic->representation);
		    current->end_block = end_block;

		    /*
		     * memory reference size
		     */
		     current->size = pointer_size ? pointer_size
						  : reg_size;
		    if (end_block)
		       flush_basic_block();

		    /* reset the register mask */
		    reg_mask = 0;
		    start_pos = save_pos();
		  };

opt_instrPrefix : instrPrefix tail
		| /* null */
		  {
			lock = FALSE;
			repeat_prefix = FALSE;
		  }
		;


asmInstruction  : MNEMONIC
		  {
			$$ = NULL;
			num_ops = 0;
		  }
		| MNEMONIC expr
		  {
			$$ = (enum addr_modes *) malloc(2 * sizeof(int));
			$$[0] = $2;
			$$[1] = UNKNOWN;
			end_first = save_pos();
			num_ops = 1;
		  }
		| MNEMONIC expr COMMA expr opt_third_operand
		  {
		      /* return an array containing the addressing */
		      /* modes of the operands */

			$$ = (enum addr_modes *) malloc(2 * sizeof(int));
			$$[0] = $2;
			$$[1] = $4;

			if (!end_second)
				/* in the presence of a third operand,
				 * end_second is already set!
				 */
				end_second = save_pos();

			num_ops = 2;
		  }
       ;

opt_third_operand : /* null */
		  | COMMA expr
		   /* some instructions like IMULL have a third
		    * operand, but that is never a memref and hence
		    * ignored */
		  ;

instrPrefix     : REP
		  {
			repeat_prefix = TRUE;
		  }
		| REPE
		  {
			repeat_prefix = TRUE;
		  }
		| REPZ
		  {
			repeat_prefix = TRUE;
		  }
		| REPNE
		  {
			repeat_prefix = TRUE;
		  }
		| REPNZ
		  {
			repeat_prefix = TRUE;
		  }
		| LOCK
		  {
			lock = TRUE;
		  }
		;

expr            : e01
		| segmentRegister
		  {
			$$ = REG;
		  }
		| register
		  {
			$$ = REG;
		  }
		;

e01             : e02
		| QWORD PTR e02
		  {
			pointer_size = 8;
			$$ = $3;
		  }
		| DWORD PTR e02
		  {
			pointer_size = 4;
			$$ = $3;
		  }
		| WORD PTR e02
		  {
			pointer_size = 2;
			$$ = $3;
		  }
		| BYTE PTR e02
		  {
			pointer_size = 1;
			$$ = $3;
		  }
		| FAR PTR e02
		  {
			pointer_size = 4;
			$$ = $3;
		  }
		| NEAR PTR e02
		  {
			pointer_size = 2;
			$$ = $3;
		  }
		| OFFSET e02
		  {
			$$ = IMMEDIATE;
		  }
		| OFFSET FLAT COLON e02
		  {
			$$ = IMMEDIATE;
		  }
		| SEG    e02
		  {
			$$ = IMMEDIATE;
		  }
		;

e02             : segmentRegister COLON e03
		  {
			$$ = $3
		  }
		| e03
		;

e03             : disp SQOB address SQCB e04
		  {
			if ($3 == REG)
			   $$ = INDIRECT;
			else
			   $$ = DIRECT;
		  }
		| disp PLUS SQOB address SQCB 
		  {
			if ($4 == REG)
			   $$ = INDIRECT;
			else
			   $$ = DIRECT;
		  }
		| SQOB address SQCB
		  {
			if ($2 == REG)
			   $$ = INDIRECT;
			else
			   $$ = DIRECT;
		  }
		| ID e05
		  {
			$$ = DIRECT;
		  }
		| constant
		  {
			$$ = IMMEDIATE;
		  }
		;

e04             : SQOB address SQCB e05
		| PLUS constant
		| /* null */
		;

e05             : PLUS constant
		| /* null */
		;

sign			: PLUS
				| MINUS
				;

address         : register
		  {
			$$ = REG;
		  }
		| constant
		  {
			$$ = IMMEDIATE;
		  }
		| address sign address
		  {
			$$ = REG;
		  }
		| address ASTERISK address
		  {
			$$ = REG;
		  }
		| address sign address sign address
		  {
			$$ = REG;
		  }
		;

disp            : constant
		  {
			$$ = IMMEDIATE;
		  }
		| ID
		  {
			$$ = DIRECT;
		  }
		;

constant        : PLUS CONSTANT
		| MINUS CONSTANT
		| CONSTANT
		;

register        : specialRegister
		| gpRegister
		  {
			reg_size = 2;
		  }
		| egpRegister
		  {
			reg_size = 4;
		  }
		| byteRegister
		  {
			reg_size = 1;
		  }
		| stackRegister
		;

specialRegister : CR0
		| CR2
		| CR3
		| TR6
		| TR7
		| ST OB CONSTANT CB;

gpRegister      : AX
		  {
		      reg_mask |= EAX_MASK;
		  }
		| BX
		  {
		      reg_mask |= EBX_MASK;
		  }
		| CX
		  {
		      reg_mask |= ECX_MASK;
		  }
		| DX
		  {
		      reg_mask |= EDX_MASK;
		  }
		| BP
		  {
		      reg_mask |= EBP_MASK;
		  }
		| DI
		  {
		      reg_mask |= EDI_MASK;
		  }
		| SI
		  {
		      reg_mask |= ESI_MASK;
		  }
		;

egpRegister     : EAX
		  {
		      reg_mask |= EAX_MASK;
		  }
		| EBX
		  {
		      reg_mask |= EBX_MASK;
		  }
		| ECX
		  {
		      reg_mask |= ECX_MASK;
		  }
		| EDX
		  {
		      reg_mask |= EDX_MASK;
		  }
		| EBP
		  {
		      reg_mask |= EBP_MASK;
		  }
		| EDI
		  {
		      reg_mask |= EDI_MASK;
		  }
		| ESI
		  {
		      reg_mask |= ESI_MASK;
		  }
		;
stackRegister   : SP
		  {
		      reg_mask |= ESP_MASK;
		      reg_size = 1;
		  }
		| ESP
		  {
		      reg_mask |= ESP_MASK;
		      reg_size = 2;
		  }
		;

byteRegister    : AL
		  {
		      reg_mask |= EAX_MASK;
		  }
		| AH
		  {
		      reg_mask |= EAX_MASK;
		  }
		| BL
		  {
		      reg_mask |= EBX_MASK;
		  }
		| BH
		  {
		      reg_mask |= EBX_MASK;
		  }
		| CL
		  {
		      reg_mask |= ECX_MASK;
		  }
		| CH
		  {
		      reg_mask |= ECX_MASK;
		  }
		| DL
		  {
		      reg_mask |= EDX_MASK;
		  }
		| DH
		  {
		      reg_mask |= EDX_MASK;
		  }
		;

segmentRegister : CS
		  {
		      reg_mask |= CS_MASK;
		  }
		| DS
		  {
		      reg_mask |= DS_MASK;
		  }
		| ES
		  {
		      reg_mask |= ES_MASK;
		  }
		| FS
		  {
		      reg_mask |= FS_MASK;
		  }
		| GS
		  {
		      reg_mask |= GS_MASK;
		  }
		| SS
		  {
		      reg_mask |= SS_MASK;
		  }
		;

opt_comment     : /* null */
		| COMMENT;

directive       : DIRECTIVE
		  ;


%%

#include "write.h"

extern FILE *yyin;
char input_filename[MAX_ID_LENGTH];
int token_count = 0;

extern char *optarg;
/* this might need changes across platforms */
extern int optind;

/* doesn't support any options yet */
int
parse_cmd_line (int argc, char **argv)
{
    int c, errflg;

    errflg = 0;
    optind = 1;

    while ((c = getopt (argc, argv, "Vcgh")) != EOF)
	switch (c) {
	    case 'V':
		default_data_movement = TRUE;
		break;
	    case 'c':
		count_references = TRUE;
		break;
	    case 'g':
		stack_ref_augment = FALSE;
		break;
	    case 'h':
		usage ();
		break;
	    case '?':
		errflg++;
		break;
	    default:
		break;
	}

    if ((default_data_movement) && (stack_ref_augment))
	yyerror ("Data movement is not supported for stack references\n"
		 "You should turn the -g flag on\n");

    if (errflg)
	usage ();
}

void
usage ()
{
    fprintf (stderr, "usage: doctor [<option> ... ] [filename]\n"
	     "Options:\n"
	     "\t-V: Turns data movement on\n"
	     "\t-c: Instrument to count the number of memory references\n"
	     "\t-g: Instrument only global references\n"
	     "\t-h: Print this help screen\n");
    exit (1);
}

main (int argc, char *argv[])
{
    extern int yylineno;
    extern char yytext[];

    parse_cmd_line (argc, argv);

    /* the last one is the input file name.. */

    if (optind == argc - 1)
	strcpy (input_filename, (const char *) argv[optind]);
    else if (argc > optind)
	fprintf (stderr, "doctor: warning: junk at the end of command"
		 " line\n");

    yyinit ();

#if YYDEBUG != 0
    yydebug = 1;
#endif

    fprintf (stdout, "; This code is generated by doctor \n");

    yyparse ();
    cleanup ();

    return 0;

}
