#ifndef _INSTR_H
#define _INSTR_H

/*
 * instr.h,v 1.1.1.1 1996/03/03 17:39:51 arsharma Exp
 * This file has the data structure definitions in which assembly
 * language instructions will be read into.
 *
 */

/* The size of the aug buffer */
#define AUG_BUF_SIZE 100000

/* The maximum size of an operand in characters */
#define MAX_OP_SIZE 40

#define SUCC_INSTR(instr_num) ((instr_num + 1) % MAX_BB_INSTR)

#define PRED_INSTR(instr_num) ((instr_num) ? (instr_num - 1) :\
					     (MAX_BB_INSTR - 1))

struct instruction_type {
    mnemonic_table_t *mnemonic; /* A pointer to the  mnemonic */
    unsigned source_line;	/* wondering if this is the c code or
				 * the asm code	      lineno ??
				 */
    unsigned ticks;

    /*
     * these are indices into the output buffer	      holding the text from
     * the end of the last instruction to the beginning of the current
     * instruction, including the non-instructions
     */
    unsigned start_text, end_text;

    /*
     * This points to the beginning of the line	      containing the
     * instruction
     */
    unsigned start_instr;

    /* indicates the start and end of the first operand	 */
    unsigned start_first, end_first;

    /*
     * indicates the start and end of the second operand in the	      output
     * buffer
     */
    unsigned start_second, end_second;

    /* pointer to the augmentation buffer, having the augmented text */
    char *aug_start;

    /*	We have some augmentation after the instruction too */
    char *post_aug_start;

	/* In NT, sizes can't be determined from the mnemonic */
	unsigned size;
    /*
     * This is the string holding the text of the operand, which is a
     * memory reference. Null, if there are none.
     */
    unsigned num_ops;		/* The number of operands */

    /* set if this is the end of the basic block */
    unsigned end_block:1;

    unsigned floating_point:1;	/* is this a float instruction ? */
    unsigned augment:1;		/* augment if this is true */
    unsigned lock:1;		/* is the instruction atomic ? */
    unsigned rep:1;		/* repeat prefix ? */
    unsigned double_data:1;	/* 8 byte data movement ? */

    /* pointer to an array of addressing modes */
    enum addr_modes *modes;

    /* keeps track of the regs touched by this instruction */
    unsigned reg_mask;

    /* finally, the most important of all */
    ref_t *mem_ref[2];
};

typedef struct instruction_type instr_t;

/* some proto types */
instr_t *new_instruction ();
int flush_basic_block ();
int cleanup ();
void process_instruction (instr_t *instr);
int check_for_aug_flags (instr_t *instr);

#endif /* INSTR_H */
