#ifndef _LEX_H
#define _LEX_H

/* lex.h,v 1.1.1.1 1996/03/03 17:39:51 arsharma Exp */

/*
 * mnemonic size field:
 * -------------------
 * Indicates the size of an operand, that can be used with the
 * mnemonic.
 * 0  indicates that no memory references are possible from this
 *    instruction
 * A positive number indicates the size in bytes
 *
 * read_write field:
 * ----------------
 *   KNOWN : familiar pattern: dest is write, src is read and by
 *	     default, a memory reference in a single operand instruction
 *	     is assumed to be a read.
 *
 *   MEMORY_READ: A memory reference is always a read
 *
 *   MEMORY_WRITE: A memory reference is always a write
 *
 *   IMPLICIT_READ, IMPLICIT_WRITE: self explanatory
 *
 *   SPECIAL: read/write more than one byte
 *   READ_WRITE: It's possible that memory is both read and written
 *   NO_REF: We're cocksure that memory is not referenced
 */

enum read_write_t {
    KNOWN = 0, MEMORY_READ, MEMORY_WRITE, IMPLICIT_READ, IMPLICIT_WRITE,
    SPECIAL, READ_WRITE, NO_REF
};

struct mnemonic_table {
    char *mnemonic_name;
    int representation;
    enum read_write_t read_write;
    unsigned clock_cycles;
    int operand_size;
};

typedef struct mnemonic_table mnemonic_table_t;

/* function prototypes */

static int screen ();
int token_store ();
char *mnem2a (int mnem);
int yyinit ();
int yywrap ();
int yywhere ();

#endif /* _LEX_H */
