#include "doctor_t.h"
#include "lex.h"
#include "memref.h"
#include "instr.h"
#include "register.h"

char *register_table[] =
{
    "%eax",
    "%ebx",
    "%ecx",
    "%edx",
    "%ebp",
    "%esi",
    "%edi",
    "%esp",
    "%cs",
    "%ss",
    "%ds",
    "%es",
    "%fs",
    "%gs",
};

char *word_register_table[] =
{
    "%ax",
    "%bx",
    "%cx",
    "%dx",
    "%bp",
    "%si",
    "%di",
    "%sp",
};

char *byte_register_table[] =
{
    "%al",
    "%bl",
    "%cl",
    "%dl"
};

/* returns an unused register of a given size */
char *
get_unused_register (instr_t *instr, int size)
{
    unsigned mask, check, offset;

    mask = instr->reg_mask;

    offset = 0;
    check = 1;

    /* shift the bits in check, until we find a bit that is not set */
    while (mask & check) {
	check = check << 1;
	offset++;
    }

    if (size == 4)
	return register_table[offset];
    else if (size == 2)
	return word_register_table[offset];
    else if ((size == 1) && (offset < 4))
	return byte_register_table[offset];
    else
	yyerror ("get_unused_register: failed to find one\n");
}
