#ifndef __queue_h
#define __queue_h

#include <stddef.h>

/* For generic queue operations, define queue node types.
 * The "next" field points to the front of the list. This is the item
 * that will be returned on a "dequeue" operation. The "prev" field
 * points to the tail of the list. The "enqueue" operation links an
 * item at the tail of the list using the "prev" pointer.
 *
 * All queues are assumed to have a head node. An empty queue must be
 * initialized so that its "next" and "prev" fields point to the head node.
 */
typedef struct qnode_t {
    struct qnode_t *next;
    struct qnode_t *prev;
} qnode_t, *qnode_ptr;

/* queue operation function prototypes
 * These functions require the next and prev fields to be the first 2 fields
 * in the structure being linked.
 */

#ifdef __cplusplus
extern "C" {
#endif

void      init_q(qnode_ptr qhead);
int       onqueue_q(qnode_ptr qhead, qnode_ptr pnode);
void      enqueue_q(qnode_ptr qhead, qnode_ptr pnew);
qnode_ptr dequeue_q(qnode_ptr qhead);
void      remove_q(qnode_ptr pnode);

/* memory management function prototypes */
qnode_ptr new_item(void *free_item_ptr, size_t item_size, char *name);
qnode_ptr new_zitem(void *free_item_ptr, size_t item_size, char *name);
void free_item(void *free_item_ptr, void *item_ptr);

#ifdef  __cplusplus
}
#endif /* __cplusplus */

#endif /* __queue_h */
